@@ -1,29 +1,27 @@
 /********************************************************************************
    zfuncs.h      include file for zfuncs functions
 
-   Copyright 2006-2016 Michael Cornelison
-   source URL:  kornelix.net
+   Copyright 2006-2017 Michael Cornelison
+   source code URL: http://kornelix.net
    contact: kornelix@posteo.de
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
+   (at your option) any later version. See http://www.gnu.org/licenses
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
    See the GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 *********************************************************************************/
 
-//     zfuncs.h   version  v.6.5
+//     zfuncs.h   version  v.6.7
 
 #include <sys/sysinfo.h>
 #include <sys/time.h>
+#include <sys/timeb.h>
 #include <sys/stat.h>
 #include <sys/resource.h>
 #include <sys/file.h>
@@ -98,7 +96,6 @@ void trace(cchar *file, cchar *func, int line, void *addr);
 void tracedump();                                                                //  dump program trace data
 
 void beroot(int argc = 0, char *argv[] = 0);                                     //  restart image as root if password OK
-
 double get_seconds();                                                            //  seconds since 2000.01.01
 void start_timer(double &time0);                                                 //  start a timer
 double get_timer(double &time0);                                                 //  get elapsed time in seconds
@@ -123,6 +120,7 @@ void zput_locked(int &param, int value);
 int  zadd_locked(int &param, int incr);                                          //  increment multi-thread parameter
 void start_detached_thread(void * tfunc(void *), void * arg);                    //  start detached thread function
 void synch_threads(int NT = 0);                                                  //  synchronize NT threads
+int  zsystem(cchar *command);                                                    //  system() + return child process status
 int  shell_quiet(cchar *command, ...);                                           //  format/run shell command, return status
 int  shell_ack(cchar *command, ...);                                             //   ""  + popup an error message if error
 int  shell_asynch(cchar *command, ...);                                          //  start shell command, return immediately
@@ -137,6 +135,8 @@ int  samedirk(cchar *file1, cchar *file2);
 int  parsefile(cchar *path, char **dir, char **file, char **ext);                //  parse a filespec
 int  renamez(cchar *file1, cchar *file2);                                        //  rename, also across file systems
 int  check_create_dir(char *path);                                               //  check if directory exists, ask to create
+int  copyFile(cchar *sfile, cchar *dfile);                                       //  copy file to file or file to directory
+char * combine_argvs(int argc, char *argv[], int Nth);                           //  combine argv[ii] elements from Nth to last
 
 //  measure CPU time spent in a function or code block within a function
 
@@ -178,7 +178,7 @@ void strToLower(char *dest, cchar *source);
 void strToLower(char *string);                                                   //  conv. string to lower case
 int  repl_1str(cchar *strin, char *strout, cchar *ssin, cchar *ssout);           //  copy string and replace 1 substring
 int  repl_Nstrs(cchar *strin, char *strout, ...);                                //  copy string and replace N substrings
-int  breakup_text(cchar *in, char **&out, cchar *dlm, int cc1, int cc2);         //  break long string into substrings for output
+int  breakup_text(cchar *in, char **&out, cchar *dlm, int cc1, int cc2);         //  break long string into substrings
 void strncpyx(char *out, cchar *in, int ccin);                                   //  conv. string to hex format
 void StripZeros(char *pNum);                                                     //  1.230000E+12 --> 1.23E+12
 int  blank_null(cchar *string);                                                  //  test for blank/null string
@@ -187,6 +187,7 @@ int  utf8len(cchar *utf8string);
 int  utf8substring(char *utf8out, cchar *utf8in, int pos, int cc);               //  get graphic substring from UTF8 string
 int  utf8_check(cchar *string);                                                  //  check utf8 string for encoding errors
 int  utf8_position(cchar *utf8in, int Nth);                                      //  get byte position of Nth graphic char.
+int  zsed(cchar *file, ...);                                                     //  replace string1/3... with string2/4...
 
 //  number conversion ===========================================================
 
@@ -217,13 +218,15 @@ typedef int HeapSortUcomp(cchar *rec1, cchar *rec2);
 void HeapSort(int vv[], int nn);                                                 //  Heap Sort - integer
 void HeapSort(float vv[], int nn);                                               //  Heap Sort - float
 void HeapSort(double vv[], int nn);                                              //  Heap Sort - double
-void HeapSort(char *vv[], int nn);                                               //  Heap Sort - char *, ascending order
-void HeapSort(char *vv1[], char *vv2[], int nn);                                 //  Heap Sort - parallel char *, ascending order
-void HeapSort(char *vv[], int nn, HeapSortUcomp);                                //  Heap Sort - char *, user-defined order
+void HeapSort(char *vv[], int nn);                                               //  Heap Sort - char, ascending order
+void HeapSort(char *vv1[], char *vv2[], int nn);                                 //  Heap Sort - parallel char *, ascending
+void HeapSort(char *vv[], int nn, HeapSortUcomp);                                //  Heap Sort - char, user-defined order
 void HeapSort(char *recs, int RL, int NR, HeapSortUcomp);                        //  Heap Sort - records, user-defined order
 
 int MemSort(char * RECS, int RL, int NR, int KEYS[][3], int NK);                 //  memory sort, records with multiple keys
 
+int zmember(int testval, int matchval1, ...);                                    //  test if value matches any in a list
+
 //  variable string list functions ==============================================
 
 struct  pvlist {
@@ -281,7 +284,7 @@ cchar * get_zdatadir();
 cchar * get_zdocdir();                                                           //  get document directory
 cchar * get_zicondir();                                                          //  get icon directory
 
-void get_hardware_info(GtkWidget *widget = 0);                                   //  get display, screen, mouse, gtksettings
+void get_hardware_info(GtkWidget *widget);                                       //  get display, screen, mouse, gtksettings
 void zsetfont(cchar *newfont);                                                   //  set new app font and size
 
 int  locale_filespec(cchar *ftype, cchar *fname, char *filespec);                //  get a locale dependent file
@@ -308,6 +311,29 @@ void zmainloop(int skip = 0);
 void zmainsleep(float secs);                                                     //  do main loop and sleep designated time
 void zthreadcrash();                                                             //  crash if thread is not main() thread
 
+//  cairo drawing region for GDK window      GTK 3.21 version
+
+#if GTK_CHECK_VERSION(3,22,0)
+
+typedef struct {
+   GdkWindow               *win;
+   cairo_rectangle_int_t   rect;
+   cairo_region_t          *reg;
+   GdkDrawingContext       *ctx;
+   cairo_t                 *dcr = 0;
+}  draw_context_t;
+
+#else
+
+typedef struct {
+   cairo_t    *dcr;
+}  draw_context_t;
+
+#endif
+
+cairo_t * draw_context_create(GdkWindow *gdkwin, draw_context_t &context);
+void draw_context_destroy(draw_context_t &context);
+
 //  text window print and read utilities
 
 void wprintx(GtkWidget *Win, int line, cchar *mess, int bold = 0);               //  write text to line, optional bold
@@ -369,7 +395,7 @@ void gmenuz(GtkWidget *parent, cchar *title, cchar *ufile, gmenuz_cbfunc);
 
 /********************************************************************************/
 
-//  create vertical menu/toolbar in vertical packing box                         //  v.5.5
+//  create vertical menu/toolbar in vertical packing box
 
 struct vmenuent {                                                                //  menu data from caller
    cchar       *name;                                                            //  menu name, text
@@ -386,7 +412,8 @@ struct vmenuent {
 };
 
 struct Vmenu {
-   GtkWidget   *vbox;                                                            //  parent window
+   GtkWidget   *vbox;                                                            //  parent window (container)
+   GtkWidget   *topwin;                                                          //  top-level window of parent
    GtkWidget   *layout;                                                          //  drawing window
    int         xmax, ymax;                                                       //  layout size
    int         mcount;                                                           //  menu entry count
@@ -408,6 +435,7 @@ void Vmenu_add(Vmenu *vbm, cchar *name, cchar *icon,
 #define zdmaxwidgets 300
 #define zdmaxbutts 10
 #define zdsentinel 0x97530000
+#define zdialog_max 20
 
 struct zwidget  {
       cchar       *type;                           //  dialog, hbox, vbox, label, entry ...
@@ -422,11 +450,12 @@ struct zwidget  {
       int         wrap;                            //  wrap mode for edit widget
       int         rescale;                         //  widget is rescaled for more resolution
       float       lval, nval, hval;                //  scale range and neutral value
+      float       lolim, hilim, step;              //  range and step value for "float" widget
       GtkWidget   *widget;                         //  GTK widget pointer
 };
 
 struct zdialog  {
-      int         sentinel1, sentinel2;            //  validity sentinels
+      int         sentinel1;                       //  validity sentinel1
       int         uniqueID;                        //  unique ID, monotone increasing              
       void        *eventCB;                        //  widget event user callback function
       int         zrunning;                        //  dialog is running (0,1)
@@ -438,6 +467,9 @@ struct zdialog  {
       cchar       *compbutton[zdmaxbutts];         //  dialog completion button labels
       GtkWidget   *compwidget[zdmaxbutts];         //  dialog completion button widgets
       zwidget     widget[zdmaxwidgets];            //  dialog widgets (EOF = type = 0)
+      char        event[40];                       //  active event or widget
+      GtkWidget   *lastwidget;                     //  last widget active
+      int         sentinel2;                       //  validity sentinel2
 };
 
 zdialog *zdialog_new(cchar *title, GtkWidget *parent, ...);                      //  create a zdialog with opt. buttons
@@ -513,15 +545,15 @@ int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent
 
 //  popup message dialogs
 
-void zmessageACK(GtkWidget *parent, cchar *format, ... );                           //  display message, wait for OK
-int  zmessageYN(GtkWidget *parent, cchar *format, ... );                            //  display message, wait for YES/NO
-zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *format, ...);           //  show message, timeout or cancel
-char * zdialog_text(GtkWidget *parent, cchar *title, cchar *inittext);              //  get short text input from user
-int  zdialog_choose(GtkWidget *parent, cchar *message, ...);                        //  show message, return choice
-void poptext_mouse(cchar *text, int dx, int dy, float s1, float s2);                //  show popup text at mouse posn
-void poptext_window(cchar *text, GtkWindow *, int mx, int my, float s1, float s2);  //  show popup text at window posn
-int  poptext_killnow();                                                             //  kill current popup window
-int  popup_image(cchar *imagefile, GtkWindow *parent, int Fnewin, int size);        //  popup window with image
+void zmessageACK(GtkWidget *parent, cchar *format, ... );                                 //  display message, wait for OK
+int  zmessageYN(GtkWidget *parent, cchar *format, ... );                                  //  display message, wait for YES/NO
+zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *format, ...);                 //  show message, timeout or cancel
+char * zdialog_text(GtkWidget *parent, cchar *title, cchar *inittext);                    //  get short text input from user
+int  zdialog_choose(GtkWidget *parent, cchar *message, ...);                              //  show message, return choice
+void poptext_mouse(GtkWidget *parent, cchar *text, int dx, int dy, float s1, float s2);   //  show popup text at mouse
+void poptext_window(cchar *text, GtkWindow *, int mx, int my, float s1, float s2);        //  show popup text at window posn
+int  poptext_killnow();                                                                   //  kill current popup window
+int  popup_image(cchar *imagefile, GtkWindow *parent, int Fnewin, int size);              //  popup window with image
 
 //  file chooser dialogs for one file or multiple files
 
@@ -534,10 +566,10 @@ void print_image_file(GtkWidget *parent, cchar *imagefile);
 
 //  drag and drop functions
 
-typedef char * drag_drop_source_func();                                          //  user function, set drag-drop text     16.08
+typedef char * drag_drop_source_func();                                          //  user function, set drag-drop text
 typedef void drag_drop_dest_func(int x, int y, char *text);                      //  user function, get drag-drop text
 void drag_drop_source(GtkWidget *window, drag_drop_source_func);                 //  connect source window to user function
-void drag_drop_dest(GtkWidget *window, drag_drop_dest_func);                     //  connect destination window to user function
+void drag_drop_dest(GtkWidget *window, drag_drop_dest_func);                     //  connect dest. window to user function
 
 //  miscellaneous GDK/GTK functions
 
